//----------------------------------------------------------------------------
// user_logic.v - module
//----------------------------------------------------------------------------
//
// ***************************************************************************
// ** Copyright (c) 1995-2007 Xilinx, Inc.  All rights reserved.            **
// **                                                                       **
// ** Xilinx, Inc.                                                          **
// ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
// ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
// ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
// ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
// ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
// ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
// ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
// ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
// ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
// ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
// ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
// ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
// ** FOR A PARTICULAR PURPOSE.                                             **
// **                                                                       **
// ***************************************************************************
//
//----------------------------------------------------------------------------
// Filename:          user_logic.v
// Version:           1.00.a
// Description:       User logic module.
// Date:              Tue Oct 30 18:45:07 2007 (by Create and Import Peripheral Wizard)
// Verilog Standard:  Verilog-2001
//----------------------------------------------------------------------------
// Naming Conventions:
//   active low signals:                    "*_n"
//   clock signals:                         "clk", "clk_div#", "clk_#x"
//   reset signals:                         "rst", "rst_n"
//   generics:                              "C_*"
//   user defined types:                    "*_TYPE"
//   state machine next state:              "*_ns"
//   state machine current state:           "*_cs"
//   combinatorial signals:                 "*_com"
//   pipelined or register delay signals:   "*_d#"
//   counter signals:                       "*cnt*"
//   clock enable signals:                  "*_ce"
//   internal version of output port:       "*_i"
//   device pins:                           "*_pin"
//   ports:                                 "- Names begin with Uppercase"
//   processes:                             "*_PROCESS"
//   component instantiations:              "<ENTITY_>I_<#|FUNC>"
//----------------------------------------------------------------------------

module user_logic
(
  // -- ADD USER PORTS BELOW THIS LINE ---------------
  // --USER ports added here 
  // -- ADD USER PORTS ABOVE THIS LINE ---------------
  LED_sig,                        // LED 
  SW_sig,                         // SW 
  // -- DO NOT EDIT BELOW THIS LINE ------------------
  // -- Bus protocol ports, do not add to or delete 
  Bus2IP_Clk,                     // Bus to IP clock
  Bus2IP_Reset,                   // Bus to IP reset
  IP2Bus_IntrEvent,               // IP to Bus interrupt event
  Bus2IP_Data,                    // Bus to IP data bus for user logic
  Bus2IP_BE,                      // Bus to IP byte enables for user logic
  Bus2IP_RdCE,                    // Bus to IP read chip enable for user logic
  Bus2IP_WrCE,                    // Bus to IP write chip enable for user logic
  IP2Bus_Data,                    // IP to Bus data bus for user logic
  IP2Bus_Ack,                     // IP to Bus acknowledgement
  IP2Bus_Retry,                   // IP to Bus retry response
  IP2Bus_Error,                   // IP to Bus error response
  IP2Bus_ToutSup                  // IP to Bus timeout suppress
  // -- DO NOT EDIT ABOVE THIS LINE ------------------
); // user_logic

// -- ADD USER PARAMETERS BELOW THIS LINE ------------
// --USER parameters added here 
// -- ADD USER PARAMETERS ABOVE THIS LINE ------------

// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol parameters, do not add to or delete
parameter C_DWIDTH                       = 32;
parameter C_NUM_CE                       = 3;
parameter C_IP_INTR_NUM                  = 1;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

// -- ADD USER PORTS BELOW THIS LINE -----------------
// --USER ports added here 
// -- ADD USER PORTS ABOVE THIS LINE -----------------
output     [7 : 0]               LED_sig;
input      [7 : 0]               SW_sig;
// -- DO NOT EDIT BELOW THIS LINE --------------------
// -- Bus protocol ports, do not add to or delete
input                                     Bus2IP_Clk;
input                                     Bus2IP_Reset;
output     [0 : C_IP_INTR_NUM-1]          IP2Bus_IntrEvent;
input      [0 : C_DWIDTH-1]               Bus2IP_Data;
input      [0 : C_DWIDTH/8-1]             Bus2IP_BE;
input      [0 : C_NUM_CE-1]               Bus2IP_RdCE;
input      [0 : C_NUM_CE-1]               Bus2IP_WrCE;
output     [0 : C_DWIDTH-1]               IP2Bus_Data;
output                                    IP2Bus_Ack;
output                                    IP2Bus_Retry;
output                                    IP2Bus_Error;
output                                    IP2Bus_ToutSup;
// -- DO NOT EDIT ABOVE THIS LINE --------------------

//----------------------------------------------------------------------------
// Implementation
//----------------------------------------------------------------------------

  // --USER nets declarations added here, as needed for user logic
  reg                                    intr_d1;
  reg                                    intr_d2;
  reg                                    intr_sig;
  reg    [2:0]                           start_dff;
  reg    [2:0]                           stop_dff;
  reg                                    start_sig;
  reg                                    stop_sig;
  reg                                    CNT_EN;
  reg   [18:0]                           count_base;
  wire   [3:0]                           count_10m;
  wire   [3:0]                           count_100m;
  wire   [3:0]                           count_1s;
  wire   [3:0]                           count_10s;
  wire                                   count_10m_carry;
  wire                                   count_100m_carry;
  wire                                   count_1s_carry;
  wire                                   count_10s_carry;
  // Nets for user logic slave model s/w accessible register example
  reg        [0 : C_DWIDTH-1]               slv_reg0;
  reg        [0 : C_DWIDTH-1]               slv_reg1;
  reg        [0 : C_DWIDTH-1]               slv_reg2;
  wire       [0 : 2]                        slv_reg_write_select;
  wire       [0 : 2]                        slv_reg_read_select;
  reg        [0 : C_DWIDTH-1]               slv_ip2bus_data;
  wire                                      slv_read_ack;
  wire                                      slv_write_ack;
  integer                                   byte_index, bit_index;

  // --USER logic implementation added here

  // ------------------------------------------------------
  // Example code to read/write user logic slave model s/w accessible registers
  // 
  // Note:
  // The example code presented here is to show you one way of reading/writing
  // software accessible registers implemented in the user logic slave model.
  // Each bit of the Bus2IP_WrCE/Bus2IP_RdCE signals is configured to correspond
  // to one software accessible register by the top level template. For example,
  // if you have four 32 bit software accessible registers in the user logic, you
  // are basically operating on the following memory mapped registers:
  // 
  //    Bus2IP_WrCE or   Memory Mapped
  //       Bus2IP_RdCE   Register
  //            "1000"   C_BASEADDR + 0x0
  //            "0100"   C_BASEADDR + 0x4
  //            "0010"   C_BASEADDR + 0x8
  //            "0001"   C_BASEADDR + 0xC
  // 
  // ------------------------------------------------------
  
  assign
    slv_reg_write_select = Bus2IP_WrCE[0:2],
    slv_reg_read_select  = Bus2IP_RdCE[0:2],
    slv_write_ack        = Bus2IP_WrCE[0] || Bus2IP_WrCE[1] || Bus2IP_WrCE[2],
    slv_read_ack         = Bus2IP_RdCE[0] || Bus2IP_RdCE[1] || Bus2IP_RdCE[2];

  // implement slave model register(s)
  always @( posedge Bus2IP_Clk )
    begin: SLAVE_REG_WRITE_PROC

      if ( Bus2IP_Reset == 1 )
        begin
          slv_reg0 <= 0;
          //slv_reg1 <= 0;
          slv_reg2 <= 0;
        end
      else
        case ( slv_reg_write_select )
          3'b100 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg0[bit_index] <= Bus2IP_Data[bit_index];
          /*
          3'b010 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg1[bit_index] <= Bus2IP_Data[bit_index];
          */
          3'b001 :
            for ( byte_index = 0; byte_index <= (C_DWIDTH/8)-1; byte_index = byte_index+1 )
              if ( Bus2IP_BE[byte_index] == 1 )
                for ( bit_index = byte_index*8; bit_index <= byte_index*8+7; bit_index = bit_index+1 )
                  slv_reg2[bit_index] <= Bus2IP_Data[bit_index];
          default : ;
        endcase

    end // SLAVE_REG_WRITE_PROC

  // implement slave model register read mux
  always @( slv_reg_read_select or slv_reg0 or slv_reg1 or slv_reg2 )
    begin: SLAVE_REG_READ_PROC

      case ( slv_reg_read_select )
        3'b100 : slv_ip2bus_data <= slv_reg0;
        3'b010 : slv_ip2bus_data <= slv_reg1;
        3'b001 : slv_ip2bus_data <= {4'h0,count_10s,4'h0,count_1s,4'h0,count_100m,4'h0,count_10m};
        default : slv_ip2bus_data <= 0;
      endcase

    end // SLAVE_REG_READ_PROC

  // ------------------------------------------------------------
  // Example code to drive IP to Bus signals
  // ------------------------------------------------------------

  assign IP2Bus_Data        = slv_ip2bus_data;
  assign IP2Bus_Ack         = slv_write_ack || slv_read_ack;
  assign IP2Bus_Error       = 0;
  assign IP2Bus_Retry       = 0;
  assign IP2Bus_ToutSup     = 0;
  assign IP2Bus_IntrEvent = intr_sig;
   
  // LED output 
  assign LED_sig  = slv_reg0[24:31];

  // SW input
  always @( posedge Bus2IP_Clk )
    begin
      if ( Bus2IP_Reset == 1 )
          slv_reg1 <= 0;
      else
          slv_reg1 <= {24'd0,SW_sig};
    end
 //
 // input sig 
 //
   
  always @( posedge Bus2IP_Clk or posedge Bus2IP_Reset )
    if ( Bus2IP_Reset == 1 )
      begin
        start_dff <= 3'b000;
        stop_dff <= 3'b000;
        start_sig <= 1'b0;
        stop_sig <= 1'b0;
      end
    else
      begin
        start_dff <= {start_dff[1:0],SW_sig[7]};
        if (start_dff[2:1]==2'b01)
          start_sig <= 1'b1;
        else
          start_sig <= 1'b0;
        stop_dff <= {stop_dff[1:0],SW_sig[6]};
        if (stop_dff[2:1]==2'b01)
          stop_sig <= 1'b1;
        else
          stop_sig <= 1'b0;
      end
       
 //
 //counter_en
 //     
  always @( posedge Bus2IP_Clk or posedge Bus2IP_Reset )
    begin
      if ( Bus2IP_Reset == 1 )
        CNT_EN <= 1'b0;    
      else
        if (start_sig == 1'b1)
          CNT_EN <= 1'b1;
        else
          if (stop_sig == 1'b1)    
            CNT_EN <= 1'b0;
          else     
            CNT_EN <= CNT_EN;
    end // always @ ( posedge Bus2IP_Clk or posedge Bus2IP_Reset )
 //
 //counter
 //     
 
  always @( posedge Bus2IP_Clk or posedge Bus2IP_Reset )
    begin
      if ( Bus2IP_Reset == 1 )
        count_base <= 19'd0;
      else
        if (start_sig == 1'b1)
          count_base <= 19'd0;
        else
          if (CNT_EN== 1'b1)
            if (count_base == 19'd499999)
            //if (count_base == 19'd9)
              count_base <= 19'd0;
            else
              count_base <= count_base +19'd1;
    end

assign base_en = ((count_base == 19'd499999)&&(CNT_EN== 1'b1))?1'b1:1'b0;
//assign base_en = ((count_base == 19'd9)&&(CNT_EN== 1'b1))?1'b1:1'b0;

count_10 c_10m(
 .CLK    ( Bus2IP_Clk ),   
 .RESET  ( Bus2IP_Reset),
 .EN     ( base_en),
 .INIT   ( start_sig),
 .COUNT  ( count_10m),
 .CARRY  ( count_10m_carry)
);

count_10 c_100m(
 .CLK    ( Bus2IP_Clk ),   
 .RESET  ( Bus2IP_Reset),
 .EN     ( count_10m_carry),
 .INIT   ( start_sig),
 .COUNT  ( count_100m),
 .CARRY  ( count_100m_carry)
);

count_10 c_1s(
 .CLK    ( Bus2IP_Clk ),   
 .RESET  ( Bus2IP_Reset),
 .EN     ( count_100m_carry),
 .INIT   ( start_sig),
 .COUNT  ( count_1s),
 .CARRY  ( count_1s_carry)
);

count_10 c_10s(
 .CLK    ( Bus2IP_Clk ),   
 .RESET  ( Bus2IP_Reset),
 .EN     ( count_1s_carry),
 .INIT   ( start_sig),
 .COUNT  ( count_10s),
 .CARRY  ( count_10s_carry)
);

  always @( posedge Bus2IP_Clk or posedge Bus2IP_Reset )
    begin
      if ( Bus2IP_Reset == 1 )
        begin
          intr_d1 <= 1'b0;
          intr_d2 <= 1'b0;
          intr_sig <= 1'b0;
        end
      else
        begin
          if ((SW_sig[6]==1'b1)||(SW_sig[7]==1'b1))
            intr_d1 <= 1'b1;
          else
            intr_d1 <= 1'b0;
          intr_d2 <= intr_d1;
          if ((intr_d1==1'b1)&&(intr_d2==1'b0))
            intr_sig <= 1'b1;
          else
            intr_sig <= 1'b0;
        end
    end // always @ ( posedge Bus2IP_Clk )
   //    
endmodule
